/* ym68api.h
 *
 * Copyright (C) 2006, bja <bja@kth.se>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __YM68_H__
#define __YM68_H__

#define YM68_STATE_STOPPED	0
#define YM68_STATE_PLAYING	1
#define YM68_STATE_PAUSED	2

#define YM68_TYPE_YM		1
#define YM68_TYPE_SC68		2
#define YM68_TYPE_SNDH		3
#define YM68_TYPE_UNKNOWN	4

typedef struct {
    const char *filename;
    const char *title;
    const char *author;
    const char *composer;
    const char *comment;
    const char *player;
    const char *type;
    int tracks;
    int track;
    int time_ms;
    int loop;
} ym68_info_t;

typedef struct {
    int type;
    int loop;
    int track;
    int state;
    int channels;
    int rate;
    int size;
    int samples;
    void *data;
} ym68_t;

ym68_t *ym68_init(void);
int ym68_quit(ym68_t *ym68);
int ym68_open(ym68_t *ym68, const char *filename);
int ym68_close(ym68_t *ym68);
int ym68_play(ym68_t *ym68, int track);
int ym68_process(ym68_t *ym68, void *buf, int n);
int ym68_pause(ym68_t *ym68);
int ym68_stop(ym68_t *ym68);
int ym68_set_track(ym68_t *ym68, int track);
int ym68_get_track(ym68_t *ym68);
int ym68_get_tracks(ym68_t *ym68);
int ym68_seek(ym68_t *ym68, int pos);
int ym68_tell(ym68_t *ym68);
int ym68_get_length(ym68_t *ym68);
int ym68_loop(ym68_t *ym68, int loop);
int ym68_info(ym68_t *ym68, ym68_info_t *info);

#endif
