/* ym68gui.c
 *
 * Copyright (C) 2006, bja <bja@kth.se>
 *
 * This program is based on Aylet.gp2x by ParkyDR.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>
#include <SDL/SDL.h>

#include "ym68sdl.h"
#include "ym68gui.h"
#include "ym68api.h"

struct time_tag tunetime;

char **ay_filenames;
int ay_num_files = 0;
int ay_file = 0;
int ay_track = 0;

char ay_title[20];

int highspeed;
int playing;
int paused;
int want_quit;
int stopafter;
int fadetime;
int use_ui;
int play_to_stdout;

static const char app_title[] = "ST-Sound GP2X";

static SDL_Surface *screen = NULL;
static SDL_Surface *bg = NULL;
static SDL_Joystick *joy = NULL;

static char font[96][8];
static int retval=1;
static int display_help=0;

static int joy_up;
static int joy_down;
static int key_down;

static int playlist_dirty = 0;
static int status_dirty_clear = 0;
static int status_dirty = 0;
static Uint32 timeout = 0;

extern ym68_info_t ym68info;

static void action(enum cb_action_tag a)
{
  /* only set retval if we get a zero return, i.e. we need to
   * stop current track.
   */
  if (!action_callback(a)) {
    retval=0;
  }
}



static void cb_doquit(void)
{
    action(cb_quit);
}

static void ui_plot(int x, int y, int col)
{
    unsigned short *ptr = (unsigned short*) screen->pixels;
    int offs = y * (screen->pitch / sizeof(unsigned short));

    /* Colour -1 = transparent */
    if (col != -1)
    {
	ptr[offs + x] = col;
    }
}

static void ui_draw_char(int x, int y, int fc, int bc, int c)
{
    int i,j,k;

    if (c >= ' ' && c < 128)
    {
	c -= ' ';
	for (i=0; i<8; i++)
	{
	    k = font[c][i];
	    for (j=0;j<8;j++)
	    {
		if (k&(0x80>>j))
		{
		    ui_plot(x+j, y+i, fc);
		}
		else
		{
		    ui_plot(x+j, y+i, bc);
		}
	    }
	}
    }
}    

static void ui_draw_string(int x, int y, int fc, int bc, const char* s)
{
    int i;
    char t[41];

    /* Truncate string to fit screen */
    strncpy(t,s,40);
    int l = strlen(s);
    if (x+(l*8) > 320)
    {
	l = (320-x)/8;
	t[l]='\0';
    }

    for (i=0; i<l; i++)
    {
	ui_draw_char(x+i*8, y, fc, bc, t[i]);
    }	
}

static int keyrelease(SDL_Event *event)
{
    status_dirty = 1;
    playlist_dirty = 1;

    switch(event->key.keysym.sym) {

    case SDLK_UP:
	joy_up = 0;
	break;

    case SDLK_DOWN:
	joy_down = 0;
	break;

    default:
	break;
    }

    return (0);
}

static int keypress(SDL_Event *event)
{
    switch(event->key.keysym.sym) {

    case SDLK_MINUS:
    case GP2X_BUTTON_VOLDOWN:
	action(cb_dec_fadetime);
	break;
    
    case SDLK_PLUS:
    case GP2X_BUTTON_VOLUP:
	action(cb_inc_fadetime);
	break;

    case SDLK_UP:
	joy_down = 0;
	joy_up = 1;
	//action(cb_prev_file);
	break;

    case SDLK_DOWN:
	joy_up = 0;
	joy_down = 1;
	//action(cb_next_file);
	break;

    case SDLK_LEFT:
	action(cb_prev_track);
	break;

    case SDLK_RIGHT:
	action(cb_next_track);
	break;

    case SDLK_b:
	if (playing && !paused)
	{
	    action(cb_stop);
	}
	else
	{
	    action(cb_play);
	}
	break;

    case SDLK_a:
	action(cb_pause);
	break;

    case SDLK_l:
	action(cb_restart);
	break;

    case SDLK_q:
    case SDLK_ESCAPE:
	cb_doquit();
	break;

    case SDLK_r:
	action(cb_highspeed);
	break;

    case SDLK_y:
	action(cb_inc_stopafter);
	break;
    
    case SDLK_x:
	action(cb_inc_fadetime);
	break;
    
    case SDLK_SPACE:
	display_help = ~display_help;
	break;
    
    default:
	return(FALSE);      /* don't stop event if not handled */
    }

    return(TRUE);
}

static int joy_down_event(SDL_Event *ev)
{
    switch(ev->jbutton.button) {

    case GP2X_BUTTON_UP:
	joy_up = 1;
	break;

    case GP2X_BUTTON_DOWN:
	joy_down = 1;
	break;
    }

    return (0);
}

static int joy_up_event(SDL_Event *ev)
{
    switch(ev->jbutton.button) {

    case GP2X_BUTTON_UP:
	joy_up = 0;
	break;

    case GP2X_BUTTON_DOWN:
	joy_down = 0;
	break;
    }

    return (0);
}

/*
 *
 *
 */
static int joybutton(SDL_Event *event)
{
    switch(event->jbutton.button) {

    case GP2X_BUTTON_VOLDOWN:
	action(cb_dec_fadetime);
	break;
    
    case GP2X_BUTTON_VOLUP:
	action(cb_inc_fadetime);
	break;

    case GP2X_BUTTON_UP:
	//action(cb_prev_file);
	break;

    case GP2X_BUTTON_DOWN:
	//action(cb_next_file);
	break;

    case GP2X_BUTTON_LEFT:
	action(cb_prev_track);
	break;

    case  GP2X_BUTTON_RIGHT:
	action(cb_next_track);
	break;

    case GP2X_BUTTON_B:
	if (playing && !paused) {
	    action(cb_stop);
	}
	else {
	    action(cb_play);
	}
	break;

    case GP2X_BUTTON_A:
	action(cb_pause);
	break;

    case GP2X_BUTTON_L:
	action(cb_restart);
	break;

    case GP2X_BUTTON_START:
	cb_doquit();
	break;

    case GP2X_BUTTON_R:
	action(cb_highspeed);
	break;

    case GP2X_BUTTON_Y:
	action(cb_inc_stopafter);
	break;
    
    case GP2X_BUTTON_X:
	action(cb_inc_fadetime);
	break;
    
    case GP2X_BUTTON_SELECT:
	display_help = ~display_help;
	break;

    default:
	return(FALSE);      /* don't stop event if not handled */
    }

    return(TRUE);
}


static void ui_initjoy(void)
{
    if (SDL_InitSubSystem (SDL_INIT_JOYSTICK) == -1)
    {
        fprintf(stderr, "Unable to initialise joystick: %s\n", SDL_GetError());
	exit(1);
    }
    
    if ( SDL_NumJoysticks() > 0)
    {
	joy = SDL_JoystickOpen(0);
    }


    if (joy)
    {
	SDL_JoystickEventState (SDL_ENABLE);	
    }
}

static void ui_initwin(void)
{
    FILE* fp;
    SDL_Surface* temp;

    SDL_ShowCursor(SDL_DISABLE);

    // Attempt to create a 320x240 window with 16 bit pixels.
    screen = SDL_SetVideoMode(320, 240, 16, SDL_SWSURFACE);

    if ( screen == NULL ) 
    {
        fprintf(stderr, "Unable to set 320x240 video: %s\n", SDL_GetError());
        exit(1);
    }

    fp = fopen("zxspec-zx.raw", "rb");
    if (fp != NULL)
    {
	fread(font, 0x300, 1, fp);
    }

    temp = SDL_LoadBMP("ym68gui.bmp");
    bg = SDL_ConvertSurface(temp, screen->format, SDL_SWSURFACE); 
    SDL_FreeSurface(temp);

    update_flag = 1;
}

static void ui_help()
{
    int n = 0;

    ui_draw_string(8, 104, 0, -1, "Help - Select for playlist");

    ui_draw_string(8, 120+n++*10, 0, -1, "Select     - Toggle playlist/help");
    ui_draw_string(8, 120+n++*10, 0, -1, "Up/Down    - Previous/next file");
    ui_draw_string(8, 120+n++*10, 0, -1, "Left/Right - Previous/next track");
    ui_draw_string(8, 120+n++*10, 0, -1, "L          - Loop track");
    ui_draw_string(8, 120+n++*10, 0, -1, "R          - Togle playlist enable");
    ui_draw_string(8, 120+n++*10, 0, -1, "A          - Pause");
    ui_draw_string(8, 120+n++*10, 0, -1, "B          - Start");
    //ui_draw_string(8, 120+n++*10, 0, -1, "X          - Cycle stop time");
    //ui_draw_string(8, 120+n++*10, 0, -1, "Y          - Cycle fade time");
    ui_draw_string(8, 120+n++*10, 0, -1, "Start      - Quit");
    ui_draw_string(8, 120+n++*10, 0, -1, "");

    ui_draw_string(8,120+n++*10,0,-1,"GUI based on Aylet.gp2x by ParkyDR.");
    ui_draw_string(8,120+n++*10,0,-1,"YM ST-Sound support by Arnaud Carre."); 
    ui_draw_string(8,120+n++*10,0,-1,"SC68/SNDH support using sc68."); 
}

static void ui_playlist()
{
    int bg;
    int fg;

    int n = 0;
    int i = ay_file-5;
    char f[39] = {0};

    ui_draw_string(8, 104, 0, -1, "Playlist - Select for help");

    if (ay_file < 5) {
	n = 5 - ay_file;
	i = 0;
    }

    for (; i<=ay_file+5 && i<ay_num_files; i++) {
	if (i == ay_file) {
	    //bg = 0x07FF;
	    bg = 0xfff0;
	    fg = 0x0000;
	}
	else {
	    fg = 0xfffff;
	    bg = -1;
	}

	strncpy(f, basename(ay_filenames[i]), 38);

	while(strlen(f) < 38) {
	    f[strlen(f)] = ' ';
	}

	ui_draw_string(8, 120 + n++*10, fg, bg, f);
    }
}


static void ui_draw_status(const char *filename, const char *misc,
                           const char *author, int track, const char *playingstr)
{
    char t[31];
    char s[31];

    sprintf(t,"%d / %d", track, ym68info.tracks);


    if (paused)
    {
	strcpy(s, "Paused ");
    }
    else if (playing)
    {
	if (ym68info.loop) {
	    strcpy(s, "Playing (Loop) ");
	}
	else if (playlist_enable) {
	    strcpy(s, "Playing (Playlist) ");
	}
	else {
	    strcpy(s, "Playing ");
	}
    }
    else
    {
	strcpy(s, "Stopped ");
    }

    if (highspeed)
    {
	strcat(s, "High-speed");
    }

    ui_draw_string(104, 4, 0x00, -1, app_title);

    char *tmp = strdupa(filename);
    ui_draw_string(32,20,0, -1, "File:");
    ui_draw_string(72,20,0, -1, basename(tmp));
    ui_draw_string(32,30,0, -1, "Misc:");
    ui_draw_string(72,30,0, -1, misc);
    ui_draw_string(16,40,0, -1, "Author:");
    ui_draw_string(72,40,0, -1, author);
    ui_draw_string(24,50,0, -1, "Track:");
    ui_draw_string(72,50,0, -1, t);
    ui_draw_string(8,60,0, -1, "Playing:");
    ui_draw_string(72,60,0, -1, playingstr);
    ui_draw_string(24,70,0, -1, "State:");
    ui_draw_string(72,70,0, -1, s);
    ui_draw_string(32,80,0, -1, "Time:");

    sprintf(t, "Stop:%02d:%02d", 
	    ym68info.time_ms/60000, (ym68info.time_ms/1000)%60);

    ui_draw_string(130, 80, 0, -1, t);

    if (display_help)
    {
	ui_help();
    }
    else
    {
	ui_playlist();
    }
}


static void ui_draw_status_timeonly(void)
{
    char t[31];
    if (tunetime.min < 99 && tunetime.sec < 59) {
	sprintf(t, "%02d:%02d", tunetime.min, tunetime.sec);
    }
    else {
	sprintf(t, "%02d:%02d", 99, 59);
    }
    ui_draw_string(72,80,0, -1, t);
}

static void update(void)
{

    SDL_Rect r;

    r.x = 0;
    r.y = 0;
    r.w = 320;
    r.h = 240;

    if (playlist_dirty) {
	r.x = 0;
	r.w = 320;
	r.y = 0;
	r.h = 240;
    }

    if (status_dirty || status_dirty_clear) {
	r.x = playlist_dirty ? r.x : 72;
	r.w = playlist_dirty ? r.w : 40;
	r.y = playlist_dirty ? r.y : 80;
	r.h = playlist_dirty ? r.h : 20;
    }

    if (playlist_dirty || status_dirty || status_dirty_clear) {
	SDL_BlitSurface(bg, &r, screen, &r);
    }

    if (status_dirty || status_dirty_clear) {
	ui_draw_status_timeonly();
    }

    if (playlist_dirty) {
	ui_draw_status(ay_filenames[ymfile],
		       ym68info.comment? ym68info.comment : " ",
		       ym68info.author ? ym68info.author : " ",
		       ym68info.track,
		       ym68info.title? ym68info.title : " ");
    }

    if (playlist_dirty || status_dirty || status_dirty_clear) {
	SDL_UpdateRect(screen, r.x, r.y, r.w, r.h);
    }

    status_dirty_clear = 0;
}


/* called per 1/50th, this deals with the usual events.
 * returns zero if we want to stop the current track.
 */
int ui_frame(void)
{
    SDL_Event event;
    retval = 1;

    if (playing && (SDL_GetTicks() > timeout)) {
	timeout = SDL_GetTicks() + 500;
	status_dirty_clear = 1;
    }

    while (SDL_PollEvent(&event)) {

	switch (event.type) {

	case SDL_KEYDOWN:
	    keypress(&event);
	    status_dirty = 1;
	    playlist_dirty = 1;
	    break;

	case SDL_KEYUP:
	    keyrelease(&event);
	    if (!joy_down) {
		status_dirty = 0;
		playlist_dirty = 0;
	    }
	    break;

	case SDL_QUIT:
	    return(0);

	case SDL_JOYBUTTONDOWN:
	    status_dirty = 1;
	    playlist_dirty = 1;
	    joybutton(&event);
	    joy_down_event(&event);
	    break;

	case SDL_JOYBUTTONUP:
	    joy_up_event(&event);
	    if (!key_down) {
		status_dirty = 0;
		playlist_dirty = 0;
	    }
	    break;
	}

    }    
    if (joy_down) {
	action(cb_next_file);
    }

    if (joy_up) {
	action(cb_prev_file);
    }

    if (update_flag) {
	status_dirty = 1;
	playlist_dirty = 1;
	update();
	update_flag = 0;
	status_dirty = 0;
	playlist_dirty = 0;
    }
    else {
	update();
    }


    /* this should be the only place retval gets modified by actions */

    return (retval);
}


/* called if playback status has changed without us knowing. */
void ui_change_notify(void)
{
}

void ui_init(int argc,char **argv)
{
    // Initialize SDL's subsystems - in this case, only video.

    if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) 
    {
	fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
	exit(1);
    }

    // Register SDL_Quit to be called at exit; makes sure things are
    // cleaned up when we quit.
    // atexit(SDL_Quit);

    ui_initjoy();
    ui_initwin();
}



void ui_end(void)
{
    return;
}
